/*
 * $Id: gnoclCanvas.c,v 1.6 2004/04/21 17:39:38 baum Exp $
 *
 * This file implements a Tcl interface to the Gnome canvas widget
 *
 * Copyright (c) 2001 - 2004 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2003-07:    Split from gnocl as separate library
 */

#include "canvas.h"
#include <string.h>
#include <ctype.h>

#include <assert.h>

Tcl_ObjCmdProc gnoclCanvasCmd;

int gnoclOptJoinStyle( Tcl_Interp *interp, GnoclOption *opt,
      GObject *obj, Tcl_Obj **ret )
{
   const char *txt[] = { "miter", "round", "bevel", NULL };
   const int types[] = { GDK_JOIN_MITER, GDK_JOIN_ROUND, GDK_JOIN_BEVEL };

   assert( sizeof( GDK_JOIN_MITER ) == sizeof( int ) );

   return gnoclOptGeneric( interp, opt, obj, "join style", txt, types, ret );
}

int gnoclOptCapStyle( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret )
{
   const char *txt[] = { "notLast", "butt", "round", "projecting", NULL };
   const int types[] = { GDK_CAP_NOT_LAST, GDK_CAP_BUTT,
          GDK_CAP_ROUND, GDK_CAP_PROJECTING };

   assert( sizeof( GDK_CAP_NOT_LAST ) == sizeof( int ) );

   return gnoclOptGeneric( interp, opt, obj, "cap style", txt, types, ret );
}

int gnoclOptXY( Tcl_Interp *interp, GnoclOption *opt, GObject *obj, 
      Tcl_Obj **ret )
{
   double coords[2];
   char bufX[32];
   char bufY[32];
   strcpy( bufX, opt->propName );
   *strchr( bufX, '?' ) = 'x';
   strcpy( bufY, opt->propName );
   *strchr( bufY, '?' ) = 'y';

   if( ret == NULL ) /* set value */
   {
      int    k;
      if( Tcl_ListObjLength( interp, opt->val.obj, &k ) != TCL_OK || k != 2 )
      {
         Tcl_SetResult( interp, "list must contain exactly 2 elements", 
               TCL_STATIC );
         return TCL_ERROR;
      }
      for( k = 0; k < 2; ++k )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, opt->val.obj, k, &tp ) != TCL_OK 
               || Tcl_GetDoubleFromObj( interp, tp, &coords[k] ) != TCL_OK )
         {
            return TCL_ERROR;
         }
      }

      g_object_set( obj, bufX, coords[0], bufY, coords[1], NULL );
   }
   else /* get value */
   {
      g_object_get( obj, bufX, &coords[0], bufY, &coords[1], NULL );
      *ret = Tcl_NewListObj( 0, NULL );
      Tcl_ListObjAppendElement( NULL, *ret, Tcl_NewDoubleObj( coords[0] ) );
      Tcl_ListObjAppendElement( NULL, *ret, Tcl_NewDoubleObj( coords[1] ) );
   }

   return TCL_OK;
}

int Gnoclcanvas_Init( Tcl_Interp *interp )
{

   /* printf( "Initializing gnocl canvas version %s\n", VERSION ); */

   if( Tcl_InitStubs( interp, "8.3", 0 ) == NULL )
      return TCL_ERROR;

   if( Tcl_PkgRequire( interp, "Gnocl", VERSION, 0 ) == NULL )
      return TCL_ERROR;

   if( Tcl_PkgProvide( interp, "GnoclCanvas", VERSION ) != TCL_OK )
      return TCL_ERROR;

   Tcl_CreateObjCommand( interp, "gnocl::canvas", gnoclCanvasCmd, NULL, NULL );
   
   return TCL_OK;
}

