/*
 * $Id: scrolledWindow.c,v 1.6 2004/04/05 18:16:39 baum Exp $
 *
 * This file implements the scrolledWindow widget
 *
 * Copyright (c) 2002 - 2004 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2003-08: added gnoclOptPadding
        12: switched from GnoclWidgetOptions to GnoclOption
   2002-02: Begin of developement
 */

#include "gnocl.h"
#include <string.h>
#include <assert.h>

static GnoclOption windowOptions[] =
{
   { "-child", GNOCL_STRING, NULL },
   { "-scrollbar", GNOCL_OBJ, NULL },
   { "-borderWidth", GNOCL_OBJ, "border-width", gnoclOptPadding }, 
   { "-data", GNOCL_OBJ, "", gnoclOptData },
   { "-name", GNOCL_STRING, "name" },
   { "-onDestroy", GNOCL_OBJ, "destroy", gnoclOptCommand }, 
   { "-sensitive", GNOCL_BOOL, "sensitive" }, 
   { "-visible", GNOCL_BOOL, "visible" }, 
   { NULL }
};

static const int childIdx     = 0;
static const int scrollbarIdx = 1;

static int configure( Tcl_Interp *interp, GtkScrolledWindow *window,
      GnoclOption options[] )
{
   /*
      FIXME: delete old child
      if( gtk_container_children( GTK_CONTAINER( parent ) ) != NULL )
   */
   if( options[childIdx].status == GNOCL_STATUS_CHANGED )
   {
      GtkWidget *childWidget = gnoclChildNotPacked( 
            options[childIdx].val.str, interp );
      if( childWidget == NULL )
         return TCL_ERROR;

      /* differentiate between widgets with and without 
         native scrolling capabilities */
      /* TODO: how to make this nicer / more portable? */
      /* FIXME 2
      if( GTK_WIDGET_CLASS( GTK_OBJECT( childWidget )->klass)
            ->set_scroll_adjustments_signal )
         gtk_container_add( GTK_CONTAINER( para->scrolledWindow ), 
               childWidget );
      else
      */
      gtk_scrolled_window_add_with_viewport( window, childWidget );
   }

   if( options[scrollbarIdx].status == GNOCL_STATUS_CHANGED )
   {
      GtkPolicyType hor, vert;
      if( gnoclGetScrollbarPolicy( interp, options[scrollbarIdx].val.obj, 
            &hor, &vert ) != TCL_OK )
         return TCL_ERROR;

      gtk_scrolled_window_set_policy( window, hor, vert );
   }

   return TCL_OK;
}

static int cget( Tcl_Interp *interp, GtkScrolledWindow *window, 
      GnoclOption options[], int idx )
{
   Tcl_Obj *obj = NULL;
   if( idx == childIdx )
   {
      GtkWidget *child = gtk_bin_get_child( GTK_BIN( window ) );
      if( child )
         obj = Tcl_NewStringObj( gnoclGetNameFromWidget( child ), -1 );
      else
         obj = Tcl_NewStringObj( "", 0 );
   }

   if( obj != NULL )
   {
      Tcl_SetObjResult( interp, obj );
      return TCL_OK;
   }

   return gnoclCgetNotImplemented( interp, options + idx );
}


static int windowFunc( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{
   static const char *cmds[] = { "delete", "configure", "cget", NULL };
   enum cmdIdx { DeleteIdx, ConfigureIdx, CgetIdx };
   GtkScrolledWindow *window = GTK_SCROLLED_WINDOW( data );
   int idx;

   if( Tcl_GetIndexFromObj( interp, objv[1], cmds, "command", 
         TCL_EXACT, &idx ) != TCL_OK )
      return TCL_ERROR;

   switch( idx )
   {
      case DeleteIdx:
            return gnoclDelete( interp, GTK_WIDGET( window ), objc, objv );

      case ConfigureIdx:
            {
               int ret = TCL_ERROR;
               if( gnoclParseAndSetOptions( interp, objc - 1, objv + 1, 
                     windowOptions, G_OBJECT( window ) ) == TCL_OK )
               {
                  ret = configure( interp, window, windowOptions );
               }
               gnoclClearOptions( windowOptions );
               return ret;
            }
            break;
      case CgetIdx:
            {
               int     idx;

               switch( gnoclCget( interp, objc, objv, G_OBJECT( window ), 
                     windowOptions, &idx ) )
               {
                  case GNOCL_CGET_ERROR:  
                           return TCL_ERROR;
                  case GNOCL_CGET_HANDLED:
                           return TCL_OK;
                  case GNOCL_CGET_NOTHANDLED:
                           return cget( interp, window, windowOptions, idx );
               }
            }

   }

   return TCL_OK;
}

int gnoclScrolledWindowCmd( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{
   int               ret;
   GtkScrolledWindow *window;
   
   if( gnoclParseOptions( interp, objc, objv, windowOptions ) 
         != TCL_OK )
   {
      gnoclClearOptions( windowOptions );
      return TCL_ERROR;
   }

   window = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new( NULL, NULL ) );
   gtk_scrolled_window_set_policy( window, GTK_POLICY_AUTOMATIC, 
         GTK_POLICY_AUTOMATIC );
   gtk_widget_show( GTK_WIDGET( window ) );

   /* set default value */
   gtk_container_set_border_width( GTK_CONTAINER( window ), GNOCL_PAD );

   ret = gnoclSetOptions( interp, windowOptions, G_OBJECT( window ), -1 );
   if( ret == TCL_OK )
      ret = configure( interp, window, windowOptions );
   gnoclClearOptions( windowOptions );

   if( ret != TCL_OK )
   {
      gtk_widget_destroy( GTK_WIDGET( window ) );
      return TCL_ERROR;
   }


   return gnoclRegisterWidget( interp, GTK_WIDGET( window ), windowFunc );
}
